/*
 *
 *   Copyright (c) International Business Machines  Corp., 2000
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 */

#include <semaphore.h>

/* Global data exported by engine.c */

extern evms_version_t engine_services_api_version;
extern evms_version_t engine_plugin_api_version;
extern evms_version_t engine_fsim_api_version;
extern evms_version_t engine_cluster_api_version;
extern evms_version_t engine_container_api_version;
extern evms_version_t engine_cluster_api_version;
extern evms_version_t engine_daemon_msg_version;

extern ece_nodeid_t current_nodeid;
extern char       * current_node_name;
extern BOOLEAN      local_focus;

extern dlist_t PluginList;
extern dlist_t DiskList;
extern dlist_t SegmentList;
extern dlist_t ContainerList;
extern dlist_t RegionList;
extern dlist_t EVMSObjectList;
extern dlist_t VolumeList;
extern dlist_t KillSectorList;
extern dlist_t VolumeDeleteList;
extern dlist_t DeactivateList;
extern dlist_t VolumeDataList;

extern rename_volume_t rename_volume_list;

extern plugin_record_t * local_disk_manager;
extern plugin_record_t * replace_plugin;
extern plugin_record_t * cluster_manager;
extern cluster_functions_t * ece_funcs;

extern int dm_control_fd;

extern BOOLEAN changes_pending; /* TRUE means some change was made to the current
                                   configuration and a commit would be needed
                                   to force the change to disk. */

extern engine_mode_t    engine_mode;
extern debug_level_t    debug_level;

extern char           * log_file_name;
extern int              log_file_fd;
extern u_char           log_buf[];
extern BOOLEAN          log_usec;
extern BOOLEAN          log_pid;
#define LOG_BUF_SIZE    (MAX_USER_MESSAGE_LEN + 1024)

extern int           lock_file_fd;
extern size_t        lock_file_size;
extern lock_file_t * lock_file;

extern sem_t       shutdown_sem;

/*
 * Global functions exported by engine.c
 */

/* External APIs provided by engine.c */

extern void evms_get_api_version(evms_version_t * version);

extern int evms_open_engine(char           * node_name,
			    engine_mode_t    mode,
                            ui_callbacks_t * callbacks,
                            debug_level_t    level,
                            char           * log_name);

extern int evms_close_engine(void);

extern BOOLEAN evms_changes_pending(void);

extern int evms_rediscover(void);

extern int evms_get_debug_level(debug_level_t * level);

extern int evms_set_debug_level(debug_level_t level);

extern int evms_get_kernel_info_level(debug_level_t * level);

extern int evms_set_kernel_info_level(debug_level_t level);

extern int evms_write_log_entry(debug_level_t level,
                                char        * module_name,
                                char        * fmt,
                                ...);

extern int evms_get_plugin_list(evms_plugin_code_t    type,
                                plugin_search_flags_t flags,
                                handle_array_t    * * plugin_list);

extern int evms_get_plugin_by_ID(plugin_id_t       plugin_ID,
                                 object_handle_t * plugin_handle);

extern int evms_get_plugin_by_name(char            * plugin_name,
                                   object_handle_t * plugin_handle);

extern int evms_update_evms_dev_tree(u_int32_t verbose_level);

extern const char * evms_strerror(int err_num);

/* Internal functions to share with other Engine source modules */

extern void timestamp(char * buf,
                      size_t len);

extern int engine_write_log_entry(debug_level_t level,
                                  char        * fmt,
                                  ...);

extern int unlock_engine(engine_mode_t mode);

#define MAX_LOCK_MESSAGE_LEN 512
extern int lock_engine(engine_mode_t  mode,
		       const char * * msg);

extern int check_engine_read_access(void);

extern int check_engine_write_access(void);

extern int get_dev_node_names(char * dir_name,
			      char * * * name_list,
			      int * count);

extern void cleanup_dev_evms_tree(void);

extern int ensure_app_handle(void *            object,
                             object_type_t     object_type,
                             object_handle_t * app_handle);

extern int make_handle_entry(ADDRESS object,
                             TAG     tag,
                             ADDRESS object_handle,
                             ADDRESS parameters);

extern int make_user_handle_array(dlist_t            list,
                                  handle_array_t * * ha);

extern int make_directory(char * dir_name,
			  evms_verbose_level_t verbose);

extern BOOLEAN is_devfs_installed(void);

extern BOOLEAN is_local_object(storage_object_t * obj);

extern BOOLEAN is_local_volume(logical_volume_t * vol);
