/*
 *   Copyright (c) International Business Machines  Corp., 2001
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or 
 *   (at your option) any later version.
 * 
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software 
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Module: aixregmgr
 * File: aixregmgr.h
 */ 

#ifndef _AIX_REGION_MANAGER_H_
#define _AIX_REGION_MANAGER_H_ 1


typedef u_int8_t boolean;

#define	AIX_NAME		"aix"

// Defines based on the AIX LVM source code.
#define AIX_MAX_LPS		65535	// max number of logical partitions allowed in an LV
#define AIX_NAMESIZ		64	// maximum size for the logical n name
#define AIX_MIN_NAMESIZ  3
#define AIX_NUM_COPIES		3	// max number of copies allowed of a logical partition
#define AIX_MAX_VGS		255
#define AIX_MAX_PVS		32
#define AIX_MAX_LVS		256
#define AIX_MAX_VGDAS		2
#define AIX_DEFAULT_MIRRORING	1
#define AIX_FIRST_MIRROR	2
#define AIX_MAX_MIRRORS		3	// AIX defines ALL copies as mirrors - 3 mirrors MAX - 1 orig and 2 copies
#define AIX_MIN_MIRROR_POOL	10
#define AIX_MIRROR_POOL_CHANGE	10
#define AIX_RELOC_LENGTH	256
#define VGSA_BT_PV		127
#define NBPI			32
#define AIX_CREATE_REGION     0
#define AIX_EXPAND_REGION     1
#define AIX_SECTOR_SIZE       512
#define SECTOR_SIZE_BITS      9

// Sector locations of AIX metadata structures
#define PSN_IPL_REC		0
#define PSN_LVM_REC		7
#define PSN_LVE_OFFSET		1
#define PSN_PVH_OFFSET		17
#define PSN_PVH_INCREMENT	17
#define AIX_MAX_PV_SECTORS  68
#define PSN_NAMELIST_OFFSET	33	// Negative offset from end of VGDA
#define MAX_PPENT_PER_SECTOR	16
#define MAX_SECTORS_LV_ENTRIES	16
#define MAXLVS_OFFSET		16
#define MAX_SECTORS_NAMELIST	32
#define READ_DATA				 1
#define WRITE_DATA				 2

#define AIX_MAX_PPS_PER_PV	543
#define AIX_DEFAULT_PP_SIZE	PP_SIZE_TO_VSECTORS(22)	// 2^22 = 4 MB
#define AIX_MIN_PP_SIZE		PP_SIZE_TO_VSECTORS(20)	// 2^20 = 1 MB
#define AIX_MAX_PP_SIZE		PP_SIZE_TO_VSECTORS(30)	// 2^30 = 1 GB

#define AIX_DEFAULT_VGSA_PSN	128
#define AIX_DEFAULT_VGSA_SIZE	8
#define AIX_DEFAULT_VGDA_PSN	136
#define AIX_DEFAULT_VGDA_SIZE	2098
#define AIX_DEFAULT_FIRST_PP_PSN	round_up(AIX_DEFAULT_VGSA_PSN \
						 + AIX_MAX_VGDAS \
						   * (AIX_DEFAULT_VGSA_SIZE \
						      + AIX_DEFAULT_VGDA_SIZE), 128)

#define AIX_MIRROR_FUNCTION_COUNT  2

#define AIX_MIRROR_FUNCTION   	         	 EVMS_Task_Plugin_Function
#define AIX_MIRROR_FUNCTION_ADD_MIRROR		(EVMS_Task_Plugin_Function + 1)
#define AIX_MIRROR_FUNCTION_SPLIT_MIRROR	(EVMS_Task_Plugin_Function + 2)

#define MIRROR_CONTINUE           1
#define NO_MIRROR                 0

// AIX unique identifiers. Four 4-byte numbers
typedef struct _unique_id {
	u_int32_t	word1;
	u_int32_t	word2;
	u_int32_t	word3;
	u_int32_t	word4;
} unique_id;


// AIX On-Disk Metadata Structures



// AIX timestamps.
typedef struct _timestruc_t {
	int32_t	tv_sec;
	int32_t	tv_nsec;
} timestruc_t;


// AIX IPL Record
// Always the first sector of an AIX physical volume
typedef struct ipl_rec_area {
	u_int32_t	IPL_record_id;	// This physical volume contains a
					// valid IPL record if and only if
					// this field contains IPLRECID
#define IPL_REC_ID	0xc9c2d4c1	// Value is EBCIDIC 'IBMA'
	char		reserved1[20];
	u_int32_t	formatted_cap;	// Formatted capacity. The number of
					// sectors available after formatting
					// The presence or absence of bad
					// blocks does not alter this value.
	char		last_head;	// THIS IS DISKETTE INFORMATION
					// The number of heads minus 1. Heads
					// are number from 0 to last_head.
	char		last_sector;	// THIS IS DISKETTE INFORMATION
					// The number of sectors per track.
					// Sectors are numbered from 1 to
					// last_sector.
	char		reserved2[6];
	u_int32_t	boot_code_length; // Boot code length in sectors. A 0
					// value implies no boot code present
	u_int32_t	boot_code_offset; // Boot code offset. Must be 0 if no
					// boot code present, else contains 
					// byte offset from start of boot
					// code to first instruction.
	u_int32_t	boot_lv_start;	// Contains the PSN of the start of
					// the BLV.
	u_int32_t	boot_prg_start;	// Boot code start. Must be 0 if no
					// boot code present, else contains
					// the PSN of the start of boot code.
	u_int32_t	boot_lv_length;	// BLV length in sectors.
	u_int32_t	boot_load_add;	// 512 byte boundary load address for
					// boot code.
	char		boot_frag;	// Boot code fragmentation flag. Must
					// be 0 if no fragmentation allowed,
					// else must be 0x01.
	char		boot_emulation;	// ROS network emulation flag
					// 0x0 => not an emul support image
					// 0x1 => ROS network emulation code
					// 0x2 => AIX code supporting ROS emul
	char		reserved3[2];
	u_int16_t		basecn_length;	// Number of sectors for base
					// customization. Normal mode.
	u_int16_t		basecs_length;	// Number of sectors for base
					// customization. Service mode.
	u_int32_t	basecn_start;	// Starting PSN value for base
					// customization. Normal mode.
	u_int32_t	basecs_start;	// Starting PSN value for base 
					// customization. Service mode.
	char		reserved4[24];
	u_int32_t	ser_code_length;// Service code length in sectors.
					// A 0 value implies no service code
					// present.
	u_int32_t	ser_code_offset;// Service code offset. Must be 0 if
					// no service code is present, else
					// contains byte offset from start of
					// service code to first instruction.
	u_int32_t	ser_lv_start;	// Contains the PSN of the start of
					// the SLV.
	u_int32_t	ser_prg_start;	// Service code start. Must be 0 if
					// service code is not present, else
					// contains the PSN of the start of
					// service code.
	u_int32_t	ser_lv_length;	// SLV length in sectors.
	u_int32_t	ser_load_add;	// 512 byte boundary load address for
					// service code.
	char		ser_frag;	// Service code fragmentation flag.
					// Must be 0 if no fragmentation
					// allowed, else must be 0x01.
	char		ser_emulation;	// ROS network emulation flag
					// 0x0 => not an emul support image
					// 0x1 => ROS network emulation code
					// 0x2 => AIX code supporting ROS emul
	char		reserved5[2];
	unique_id	pv_id;		// The unique identifier for this
					// physical volume.
	char		dummy[512 - 128 - sizeof(unique_id)];
} aix_ipl_rec_t;


// AIX LVM Record
// Always sector 7 (PSN_LVM_REC) of an AIX physical volume
typedef struct AIXlvm_rec_s {
	u_int32_t	lvm_id;		// LVM id field which identifies whether the PV is a member of a volume group
#define AIX_LVM_LVMID	0x5F4C564D	// LVM id field of ASCII "_LVM"

	unique_id	vg_id;		// The id of the volume group to which this physical volume belongs
	u_int32_t	lvmarea_len;	// The length of the LVM reserved area
	u_int32_t		vgda_len;	// Length of the volume group descriptor area
	int32_t		vgda_psn[2];	// The physical sector numbers of the beginning of the volume group descriptor area copies on this disk 
	int32_t		reloc_psn;	// The physical sector number of the beginning of a pool of blocks
				        // (located at the end of the PV) which are reserved for the relocation of bad blocks
	u_int32_t	reloc_len;	// The length in number of sectors of the pool of bad block relocation blocks
	u_int16_t	pv_num;	    	// The physical volume number within the volume group of this physical volume
	u_int16_t	pp_size;    	// The size in bytes for the partition, expressed as a power of 2 (i.e., the partition size is 2 to the power pp_size)
	u_int32_t	vgsa_len;	// Length of the volume group status area
	int32_t		vgsa_psn[2];	// The physical sector numbers of the beginning of the volume group status area copies on this disk
	u_int16_t	version;	// The version number of this volume group descriptor and status area
#define LVM_VERSION_1		1	// first version - AIX 3.0
#define LVM_STRIPE_ENHANCE	2	// version with striped lv's - AIX 4.1
#define LVM_1024_PPSIZE		3	// ppsizes of 512 and 1024
#define LVM_GT_1016		4	    // version with support for > 1016 pps/pv
#define	LVM_MAX_VERSION		LVM_GT_1016 // max version #

	char res1 [450];		    // reserved area
} aix_lvm_rec_t;



// AIX Volume Group Status Area (VGSA)
typedef struct _vgsa_area {
	timestruc_t	b_tmstamp;	// Beginning timestamp
	u_int32_t	pv_missing[(AIX_MAX_PVS + (NBPI -1)) / NBPI];	// Bit per PV
	unsigned char	stalepp[AIX_MAX_PVS][VGSA_BT_PV];
	int16_t		factor;
	char		resv[10];	// Padding
	timestruc_t	e_tmstamp;	// Ending timestamp
} vgsa_area;


// AIX Volume Group Header
// Beginning of the Volume Group Descriptor Area (VGDA)
typedef struct _vg_header {
	timestruc_t	vg_timestamp;	// Time of last update
	unique_id	vg_id;		// Unique id for volume group
	int16_t		numlvs;		// Number of lvs in vg
	int16_t		maxlvs;		// Max number of lvs allowed in vg
	int16_t		pp_size;	// Size of pps in the vg
	int16_t		numpvs;		// Number of pvs in the vg
	int16_t		total_vgdas;	// Number of copies of vg
					// descriptor area on disk
	int16_t		vgda_size;	// Size of volume group descriptor
	int16_t		bigvg;
	int16_t		quorum;
	int16_t		auto_varyon;
	int32_t		checksum;
	int32_t		bigda_size;
} vg_header;
 

// AIX Logical Volume Entry
// Array of LV Entries immediately follows the VG header.
typedef struct _lv_entries {
	int16_t		lvname;		// Name of LV
	int16_t		res1;		// Reserved area
	int32_t		maxsize;	// Maximum number of partitions allowed
	char		lv_state;	// State of logical volume
	char		mirror;		// None, single, or double
	int16_t		mirror_policy;	// Type of writing used to write
	u_int32_t	num_lps;	// Number of logical partitions on the lv - Base 1
	char		permissions;	// Read-write or read-only
	char		bb_relocation;	// Specifies if bad block
					// relocation is desired
	char		write_verify;	// Verify all writes to the LV
	char		mirwrt_consist;	// Mirror write consistency flag
	u_int16_t	stripe_exp;	// Stripe size in exponent value
	u_int16_t	striping_width;	// Stripe width
	u_int16_t	lv_avoid;
	u_int16_t	child_minor_num;
	char		res4[4];	// Reserved area on disk
} lv_entries;

// lv_entries->lv_state
#define LVM_LVUNDEF	0		// The LV is not defined to a volume group
#define LVM_LVDEFINED	1		// The LV is defined to a volume group 
#define LVM_LVSTALE	2		// The LV has stale logical partitions
#define LVM_LVMIRBKP	4		// The LV is an online mirror backup

// lv_entries->mirror
#define LVM_NOMIRROR	1		// The LV has only one copy
#define LVM_SINGLEMIR	2		// The LV has two copies 
#define LVM_DOUBLEMIR	3		// The LV has three copies
#define LVM_ADDMIRROR   1   		// Add a mirror copy to the LV

// lv_entries->mirror_policy
#define LVM_SEQUENTIAL	1		// Mirrors are written and read sequentially
#define LVM_PARALLEL	2		// Mirrors are written and read in parallel 
#define LVM_STRIPED	5		// Mirrors are written in

// lv_entries->permissions
#define LVM_RDWR	1       	// The LV is read/write
#define LVM_RDONLY	2       	// The LV is read only
	
// lv_entries->bb_relocation
#define LVM_RELOC	1		// Bad block relocation is performed
#define LVM_NORELOC	2		// No bad block relocation

// lv_entries->write_verify
#define LVM_VERIFY	1		// Write verification is desired
#define LVM_NOVERIFY	2		// No write verification is desired

// lv_entries->mirwrt_consist
#define LVM_CONSIST	1		// Keep consistency on
#define LVM_NOCONSIST   2		// Turn consistency off


// AIX Physical Volume Header
// First PV Header starts immediately after the array of LV entries.
// Each PV header is immeidately followed by the PP map for that PV.
typedef struct _pv_header {
	unique_id	pv_id;		// Unique identifier of PV
	u_int16_t	pp_count;	// Number of physical partitions
					// on PV
	char		pv_state;	// State of physical volume
	char		res1;		// Reserved area on disk
	int32_t		psn_part1;	// Physical sector number of 1st pp
	int16_t		pvnum_vgdas;	// Number of vg descriptor areas
					// on the physical volume
	int16_t		pv_num;		// PV number
	int32_t		res2;		// Reserved area on disk
} pv_header;
 
// pv_header->pv_state
#define LVM_PVACTIVE	1		// The PV is an active member of a VG
#define LVM_PVMISSING	2		// The PV is missing
#define LVM_PVREMOVED	4		// The PV is removed from a volume group
#define LVM_PVNOALLOC	8		// The PV does not allow allocation
#define LVM_PVSTALE	16		// The PV has stale data


// AIX Physical Partition Entry.
// PP entries immediately follow the PV header of the PV they belong to.
typedef struct _pp_entries {
	int16_t	lv_index;		// Index to lv pp is on
	int16_t	res_1;			// Reserved area on disk
	int32_t	lp_num;			// Logical part. number
	char	copy;			// The copy of the logical partition
					// that this pp is allocated for
	char	pp_state;		// Current state of pp
	char	fst_alt_vol;		// PV where partition allocation for
					// first mirror begins
	char	snd_alt_vol;		// PV where partition allocation for
					// second mirror begins
	int16_t	fst_alt_part;		// Partition to begin first mirror
	int16_t	snd_alt_part;		// Partition to begin second mirror 
	u_int64_t	res_3;		// Reserved area on disk
	u_int64_t	res_4;		// Reserved area on disk
} pp_entries;

// pp_entries->pp_state
#define LVM_PPFREE	0	// The PP is free, not allocated
#define LVM_PPALLOC	1	// The PP is allocated
#define LVM_PPSTALE	2	// The PP has stale data
#define LVM_PPRESYNC	4	// The PP is being resynced
#define LVM_PPVGDAUSE	8	// The PP is used for VGDA
#define LVM_PPEXPAND   16   // The PP is to be added to LV

// pp_entries->copy
#define LVM_PRIMARY	0	// Primary copy of an LP with no other copies
#define LVM_PRIMOF2	1	// Primary copy of an LP with two copies 
#define LVM_PRIMOF3	2	// Primary copy of an LP with three copies
#define LVM_SCNDOF2	3	// Secondary copy of an LP with two copies
#define LVM_SCNDOF3	4	// Secondary copy of an LP with three copies
#define LVM_TERTOF3	5	// Tertiary copy of an LP with three copies

#define AIX_MIRROR_ONE     0
#define AIX_MIRROR_TWO     1
#define AIX_MIRROR_THREE   2


// List of names of AIX Logical Volumes
// 32 sectors immediately before the VG trailer.
typedef struct _namelist {
	char	name[AIX_MAX_LVS][AIX_NAMESIZ];
} namelist;
 

// AIX Volume Group Trailer
// Last sector of the VGDA.
typedef struct _vg_trailer {
	timestruc_t	timestamp;	// Time of last update
	int16_t		concurrency;	// MS Nibble = concurrent capable
					// LS Nibble = concurrent auto-varyon
	int16_t		res_2;		// Reserved area on disk
	int32_t		res_3;		// Reserved area on disk
	u_int64_t	res_4;		// Reserved area on disk
	u_int64_t	res_5;		// reserved area on disk
} vg_trailer;


// End of AIX metadata structures.


// AIX Region Manager internal data structures.


// Parent structure for all VGDA metadata structures.
typedef struct aix_vgda_s {
	vg_header	* vg_head;
	lv_entries	* lv_array;
	pv_header	* pv_headers[AIX_MAX_PVS];
	pp_entries	* pp_array[AIX_MAX_PVS];
	namelist	* lv_names;
	vg_trailer	* vg_tail;
} aix_vgda_t;


// Structure describing one logical-to-physical-partition mapping.
typedef struct aix_lp_map_entry_s {
	storage_object_t	* object;
	unsigned long		pp_number;
	unsigned long		flags;
} aix_lp_map_entry_t;

#define AIX_LP_MISSING	(1 << 0)


// Private data describing an AIX LV.
typedef struct aix_region_data_s {
	lv_entries		* lv;
	aix_lp_map_entry_t	* lp_map;
	unsigned int		flags;
} aix_region_data_t;

#define AIX_REGION_EXPORTED	(1 << 0)
#define AIX_REGION_INCOMPLETE	(1 << 1)


// Private data describing an AIX PV.
typedef struct aix_pv_data_s {
	aix_ipl_rec_t		* ipl;
	aix_lvm_rec_t		* lvm;
	vg_header		* vg_head[2];
	vg_trailer		* vg_tail[2];
	pv_header		* pv_head;
	pp_entries		* pp_map;
	unsigned long		pv_state;	// PV_STATE_*
	unsigned long		flags;
} aix_pv_data_t;

#define AIX_PV_STATE_VALID		0	// Both VGDAs are valid and match.
#define AIX_PV_STATE_FIRST_VGDA		1	// Only the first VGDA is valid.
#define AIX_PV_STATE_SECOND_VGDA	2	// Only the second VGDA is valid.
#define AIX_PV_STATE_EITHER_VGDA	-1	// Both VGDAs are valid, but do not match each other.


// Private data describing an AIX VG.
typedef struct aix_container_data_s {
	aix_vgda_t		* vgda;
	vgsa_area		* vgsa;
	storage_object_t	* freespace;
	storage_object_t	* regions[AIX_MAX_LVS];
	unsigned long		pp_count;
	unsigned long		flags;		// AIX_CONTAINER_*
} aix_container_data_t;

#define AIX_CONTAINER_INVALID_VG_HEADER	(1 << 0)


// Structure for region creation options
typedef struct aix_region_options_s {
	unsigned char	name[AIX_NAMESIZ];
	unsigned long	num_lps;
    u_int32_t       add_size;
    unsigned long   add_extents;
} aix_region_options_t;

// Global symbols exported from aixregmgr.c
extern plugin_record_t * evms_plugin_records[];


// Global data used by aixregmgr
extern engine_functions_t		* aix_engine;
extern plugin_record_t			AIX_Plugin;
extern plugin_record_t			* aix_plugin;
extern dlist_t				aix_container_list;



/********** Helpful macros and inlines **********/


// Log message macros
#define MESSAGE(msg, args...)		aix_engine->user_message(aix_plugin, NULL, NULL, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_CRITICAL(msg, args...)	aix_engine->write_log_entry(CRITICAL,	aix_plugin, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_SERIOUS(msg, args...)	aix_engine->write_log_entry(SERIOUS,	aix_plugin, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_ERROR(msg, args...)		aix_engine->write_log_entry(ERROR,	aix_plugin, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_WARNING(msg, args...)	aix_engine->write_log_entry(WARNING,	aix_plugin, "%s: " msg, __FUNCTION__ , ## args)
#define LOG(msg, args...)		aix_engine->write_log_entry(DEFAULT,	aix_plugin, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_DETAILS(msg, args...)	aix_engine->write_log_entry(DETAILS,	aix_plugin, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_DEBUG(msg, args...)		aix_engine->write_log_entry(DEBUG,	aix_plugin, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_EXTRA(msg, args...)		aix_engine->write_log_entry(EXTRA,	aix_plugin, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_PROC(msg, args...)		aix_engine->write_log_entry(ENTRY_EXIT,	aix_plugin, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_EVERYTHING(msg, args...)	aix_engine->write_log_entry(EVERYTHING,	aix_plugin, "%s: " msg, __FUNCTION__ , ## args)

#define LOG_ENTRY			LOG_PROC("Enter.\n")
#define LOG_EXIT(x)			LOG_PROC("Exit. rc = %d\n", x)
#define RETURN(x)			do { LOG_EXIT(x); return(x); } while (0)

#define GET_PHYSICAL_PART_SIZE(v1) (1 << v1)

// AIX Unique IDs are four 32-bit integers
#define COMPARE_UNIQUE_IDS(id1, id2)	( (id1).word1 == (id2).word1 && \
					  (id1).word2 == (id2).word2 && \
					  (id1).word3 == (id2).word3 && \
					  (id1).word4 == (id2).word4 )
// AIX timestamps are two 32-bit integers
#define COMPARE_TIMESTAMPS(t1, t2)	( (t1).tv_sec  == (t2).tv_sec && \
					  (t1).tv_nsec == (t2).tv_nsec )

// PP Size is stored as an exponent in the AIX metadata. Need macros to
// switch between the exponent value and number of vsectors.
#define PP_SIZE_TO_VSECTORS(pp_size)	(1 << ((pp_size) - EVMS_VSECTOR_SIZE_SHIFT))
#define VSECTORS_TO_PP_SIZE(vsectors)	(aix_log2(vsectors) + EVMS_VSECTOR_SIZE_SHIFT)


// Macro to calculate the number of sectors available on an object
// for allocating to PPs.
#define AVAILABLE_PP_SECTORS(object)	((object)->size - AIX_DEFAULT_FIRST_PP_PSN - AIX_RELOC_LENGTH)


// The "size" paramaters in these three inlines MUST be power-of-2.
static inline ulong round_down(ulong n, ulong size) {
	size--;
	return(n & ~size);
}

static inline ulong round_up(ulong n, ulong size) {
	size--;
	return((n + size) & ~size);
}

static inline ulong div_up(ulong n, ulong size) {
	return(round_up(n, size) / size);
}


// Calculate the log-base2 of the argument
static inline long aix_log2( long arg )
{
	long	result = -1;
	long	tmp;

	if ( arg ) {
		tmp = arg;
		result++;
		while ( ! (tmp & 1) ) {
			result++;
			tmp >>= 1;
		}
		if ( tmp != 1 ) {
			// arg isn't a power of 2!
			result = -2;
		}
	}
	return result;
}


// Include all other AIX header files here.
#include "aix_containers.h"
#include "aix_dlist.h"
#include "aix_io.h"
#include "aix_misc.h"
#include "aix_options.h"
#include "aix_pv.h"
#include "aix_regions.h"
#include "aix_info.h"
#include "aix_dm.h"

#endif

