
/*
 *
 *   Copyright (c) International Business Machines  Corp., 2000
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *   Module: sparse
 *
 */

#ifndef SPARSE_HEADER
#define SPARSE_HEADER

#define	EVMS_SPARSE_VERSION_MAJOR 0
#define EVMS_SPARSE_VERSION_MINOR 2
#define EVMS_SPARSE_VERSION_PATCHLEVEL 0

#define EVMS_SPARSE_FEATURE_ID	4

#define EVMS_SPARSE_SIGNATURE 0x72417053 /* spAr */
#define MAX_HASH_CHAIN_ENTRIES 10

/* Segment Manager Flags */
#define  SEG_CFLAG_TOP_SEGMENT 0x00000001

/* Private flags */
#define SPARSE_KERNEL_ACTIVE  0x001
#define SPARSE_INITIALIZE_COW 0x002

/* Status flags */
#define EVMS_SPARSE 0x001
#define EVMS_SPARSE_FULL 0x008

/* plugin specific functions */
#define SPARSE_FUNCTION_COUNT 1
#define SPARSE_FUNCTION_RESET EVMS_Task_Plugin_Function

/* Private ioctl commands */ 
#define SPARSE_QUERY_PERCENT_FULL 1
#define SPARSE_CHECK_STATE 2

/* direct ioctls */
#define EVMS_SPARSE_INIT_IO 1
#define EVMS_SPARSE_REMAP 2

#define SPARSE_DEFAULT_CHUNK_SIZE 128	/* sectors == 64k */
#define SPARSE_MIN_CHUNK_SIZE 16	/* 8kB */
#define SPARSE_MAX_CHUNK_SIZE 2048	/* 1MB */

#define SPARSE_DEFAULT_VIRTUAL_SIZE 0x80000000	
#define SPARSE_MIN_VIRTUAL_SIZE	2048*1024	
#define SPARSE_MAX_VIRTUAL_SIZE	0xffffffffffffffff  

/* Option definitions */
#define SPARSE_OPTION_VIRTUALSIZE_NAME "virtualsize"  // virtualsize
#define SPARSE_OPTION_VIRTUALSIZE_INDEX	0            // virtualsize
#define SPARSE_OPTION_CHUNKSIZE_NAME "chunksize"  // chunksize
#define SPARSE_OPTION_CHUNKSIZE_INDEX 1           // chunksize
#define SPARSE_ASSIGN_OPTIONS 2
#define SPARSE_EXPAND_OPTIONS 1
#define SPARSE_SHRINK_OPTIONS 1

typedef struct sparse_metadata {
/* 0*/	u_int32_t  signature;
/* 4*/	u_int32_t  CRC;
/* 8*/	evms_version_t version;	/* structure version */
/*20*/	u_int32_t flags;
/*24*/  u_int64_t virtual_size;
/*32*/  u_int64_t child_object_size; // for safety, size of the child 
/*40*/	u_int64_t lba_of_COW_table;
/*48*/	u_int64_t lba_of_first_chunk;
/*56*/	u_int32_t chunk_size; // in sectors
/*60*/	u_int32_t total_chunks;
/*64*/  char uuid[EVMS_NAME_SIZE+1];
/*192*/ u_int8_t pad[318];
/*512*/
} sparse_metadata_t;

typedef struct sparse_hash_entry {
    u_int64_t org_chunk;
    u_int64_t sparse_chunk;
    struct sparse_hash_entry * next;
    struct sparse_hash_entry * prev;
} sparse_hash_entry_t;

typedef struct sparse_volume { //private data on data segment
    // segment manager common area
    u_int32_t signature; 
    u_int32_t cflags;
    //----------------------------
    storage_object_t * object; // my metadata object
    storage_object_t * child_object; // our consumed object
} sparse_volume_t;

typedef struct sparse_metavolume { //private data on metadata segment
    // segment manager common area
    u_int32_t signature; 
    u_int32_t cflags;
    //----------------------------
    storage_object_t * object; // my data object
    storage_object_t * child_object; // our child object
    sparse_hash_entry_t ** sparse_map;
    unsigned long chunk_size;
    unsigned long chunk_shift;
    unsigned long num_chunks; // in this volume
    unsigned long next_cow_entry; // Index into current COW table

    // LOGICAL sector of current COW table
    unsigned long long	current_cow_sector; 
    unsigned long next_free_chunk; // index of next free chunk (not LBA!)
    u_int64_t cow_table[64]; // Pointer to one sector's worth of COW tables
    unsigned long hash_table_size; // size of the hash table for the remap
    sparse_metadata_t meta_data; // copy of metadata 
    u_int32_t flags;
} sparse_metavolume_t;

typedef struct _guid_s {
    u_int32_t       time_low;              // timestamp - low order 32 bits
    u_int16_t       time_mid;              // timestamp - mid 16 bits
    u_int16_t       time_high;             // timestamp - high 16 bits
    u_int8_t        clock_seq_high;        // clock - high order 8 bits
    u_int8_t        clock_seq_low;         // clock - low order 8 bits
    u_int8_t        node[6];               // node - spatial reference - unique identification like
} __attribute__((packed)) guid_t;          //        the mac address of this systems network card.

/*
 *  Discovery Parm Block
 */
typedef struct segment_discovery_parms_s {

     dlist_t output_objects; /* move all objects here */

     int *object_count; /* total objects discovered */

} discovery_parm_block_t;

int             PluginInit( plugin_record_t *pPlugRec );
void            PluginCleanup( plugin_record_t *pPlugRec );
extern          plugin_record_t *sparse_plugin_rec;


// Log message macros
#define LOG_CRITICAL(msg, args...)	SparseEngFuncs->write_log_entry(CRITICAL,	sparse_plugin_rec, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_SERIOUS(msg, args...)	SparseEngFuncs->write_log_entry(SERIOUS,	sparse_plugin_rec, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_ERROR(msg, args...)		SparseEngFuncs->write_log_entry(ERROR,		sparse_plugin_rec, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_WARNING(msg, args...)	SparseEngFuncs->write_log_entry(WARNING,	sparse_plugin_rec, "%s: " msg, __FUNCTION__ , ## args)
#define LOG(msg, args...)		SparseEngFuncs->write_log_entry(DEFAULT,	sparse_plugin_rec, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_DETAILS(msg, args...)	SparseEngFuncs->write_log_entry(DETAILS,	sparse_plugin_rec, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_DEBUG(msg, args...)		SparseEngFuncs->write_log_entry(DEBUG,		sparse_plugin_rec, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_EXTRA(msg, args...)		SparseEngFuncs->write_log_entry(EXTRA,		sparse_plugin_rec, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_PROC(msg, args...)		SparseEngFuncs->write_log_entry(ENTRY_EXIT,	sparse_plugin_rec, "%s: " msg, __FUNCTION__ , ## args)
#define LOG_EVERYTHING(msg, args...)	SparseEngFuncs->write_log_entry(EVERYTHING,	sparse_plugin_rec, "%s: " msg, __FUNCTION__ , ## args)

#define SET_STRING(a,b) a = SparseEngFuncs->engine_alloc( strlen(b)+1 );if (a ) { strcpy(a, b); } else { rc = ENOMEM; LOG_EXIT(rc);}

#define	LOGENTRY()			LOG_PROC("Enter.\n")
#define	LOGEXIT()			LOG_PROC("Exit.\n")
#define LOG_EXIT(x) 			LOG_PROC("Exit. rc = %d\n", x)
#define RETURN(x)			{LOG_EXIT(x); return(x);}

#endif






